package uprrp.tania.activities;

import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.Toast;

import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;

import com.google.android.gms.tasks.OnSuccessListener;
import com.google.firebase.iid.FirebaseInstanceId;
import com.google.firebase.iid.InstanceIdResult;

import uprrp.tania.R;
import uprrp.tania.networking.SendExperienceRegistration;
import uprrp.tania.URLEventListener;

public class ExperienceRegistrationActivity  extends AppCompatActivity {

    private static final String TAG = "ExperienceRegistrationActivity";
    private String DEVICE_TOKEN;
    private String EXPERIENCE_ID;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {

        // Start constructor and fetch UI components
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_experience_registration);
        final Button experienceRegistrationButton = findViewById(R.id.buttonEnterExperience);

        // Change all caps text to normal capitalization
        // TODO: this is a workaround I found, any other acceptable solution is welcome
        experienceRegistrationButton.setTransformationMethod(null);

        // Request to Firebase...
        // TODO: Should we implement retry logic in onFailureListener?
        FirebaseInstanceId.getInstance().getInstanceId().addOnSuccessListener(new OnSuccessListener<InstanceIdResult>() {
            @Override
            public void onSuccess(InstanceIdResult instanceIdResult) {

            // Get device's token
            // IMPORTANT: must come first before calling
            // another function that uses the token
            DEVICE_TOKEN = instanceIdResult.getToken();

            // Set onClick listener on button
            experienceRegistrationButton.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    sendRegistrationRequest();
                }
            });

            }
        });

        // Fetch experience ID from URL (only if we launched this Activity from Browser)
        Intent intent = getIntent();
        if(Intent.ACTION_VIEW.equals(intent.getAction())) {
            Uri uri = intent.getData();
            assert uri != null; // TODO: figure out if this causes crashes...
            EXPERIENCE_ID = uri.getQueryParameter("id");
        }

    }

    private void sendRegistrationRequest() {

        // Initiate progress dialog
        // TODO: find substitute for this deprecated dialog box
        final ProgressDialog progressDialog = ProgressDialog.show(ExperienceRegistrationActivity.this,
                "Registering in Experience",
                "This shouldn't take long");

        // Send registration request to server
        SendExperienceRegistration experienceRegistrationTask = new SendExperienceRegistration(new URLEventListener() {
            @Override
            public void onSuccess() {
                progressDialog.dismiss();
                Context context = ExperienceRegistrationActivity.this;
                Intent intent = new Intent(context, MainActivity.class);
                intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
                context.startActivity(intent);
                Toast.makeText(getApplicationContext(), "You have been registered!", Toast.LENGTH_LONG).show();
            }

            @Override
            public void onFailure(Exception e) {
                progressDialog.dismiss();
                Toast.makeText(ExperienceRegistrationActivity.this, "Oops! Something went wrong...\nPlease try again", Toast.LENGTH_LONG).show();
                Log.e(TAG, "Error occurred while sending registration request to server...");
                e.printStackTrace();
            }
        });

        experienceRegistrationTask.execute(this.DEVICE_TOKEN, this.EXPERIENCE_ID);

        // UNCOMMENT THIS FOR TESTING (REMEMBER TO COMMENT .execute() LINE)
        /*
        Handler handler = new Handler();
        handler.postDelayed(new Runnable() {
            public void run() {
                progressDialog.dismiss();
                if(false) { // Imitate success
                    Context context = ExperienceRegistrationActivity.this;
                    Intent intent = new Intent(context, MainActivity.class);
                    intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
                    context.startActivity(intent);
                    Toast.makeText(getApplicationContext(), "You have been registered!", Toast.LENGTH_LONG).show();
                } else { // Imitate failure
                    Toast.makeText(ExperienceRegistrationActivity.this, "Oops! Something went wrong...", Toast.LENGTH_LONG).show();
                }
            }
        }, 5000);
        */
        // UNCOMMENT THIS FOR TESTING (REMEMBER TO COMMENT .execute() LINE)

    }

}
