package uprrp.tania.fragments;

import android.app.Activity;
import android.app.ProgressDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.TextView;
import android.widget.Toast;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;
import androidx.fragment.app.Fragment;

import com.google.android.gms.tasks.OnSuccessListener;
import com.google.firebase.iid.FirebaseInstanceId;
import com.google.firebase.iid.InstanceIdResult;

import org.researchstack.backbone.StorageAccess;
import org.researchstack.backbone.answerformat.AnswerFormat;
import org.researchstack.backbone.answerformat.ChoiceAnswerFormat;
import org.researchstack.backbone.answerformat.TextAnswerFormat;
import org.researchstack.backbone.model.Choice;
import org.researchstack.backbone.result.StepResult;
import org.researchstack.backbone.result.TaskResult;
import org.researchstack.backbone.step.InstructionStep;
import org.researchstack.backbone.step.QuestionStep;
import org.researchstack.backbone.step.Step;
import org.researchstack.backbone.task.OrderedTask;
import org.researchstack.backbone.ui.ViewTaskActivity;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

import uprrp.tania.networking.SendAnswers;
import uprrp.tania.models.AnsweredQuestionModel;
import uprrp.tania.models.AnsweredAssessmentModel;
import uprrp.tania.networking.FetchAssessment;
import uprrp.tania.networking.FetchUserStatus;
import uprrp.tania.models.AssessmentModel;
import uprrp.tania.models.QuestionModel;
import uprrp.tania.R;
import uprrp.tania.URLEventListener;
import uprrp.tania.models.UserStatusModel;

public class AssessmentsFragment extends Fragment {

    // These variables came from the sample app provided to explain the use of Research Stack
    private static final int REQUEST_SURVEY = 1;
    private static final String INSTRUCTION = "identifier";
    private static final String SAMPLE_SURVEY = "sample_assessment";

    // Our variables
    private static final String TAG = "AssessmentsFragment";
    private String DEVICE_TOKEN;
    private String ASSESSMENT_ID;
    private View thisFragment;

    @Nullable
    @Override
    public View onCreateView(LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {

        // Get reference for this fragment and other components
        this.thisFragment = inflater.inflate(R.layout.fragment_assessments, container, false);
        final Button surveyButton = thisFragment.findViewById(R.id.surveyButton);
        final Button refreshButton = thisFragment.findViewById(R.id.refreshButton);

        // Change all caps text to normal capitalization
        // TODO: this is a workaround I found, any other acceptable solution is welcome
        surveyButton.setTransformationMethod(null);
        refreshButton.setTransformationMethod(null);

        // Request to Firebase...
        // TODO: Should we implement retry logic in onFailureListener?
        FirebaseInstanceId.getInstance().getInstanceId().addOnSuccessListener(new OnSuccessListener<InstanceIdResult>() {
            @Override
            public void onSuccess(InstanceIdResult instanceIdResult) {

                // Get device's token
                // IMPORTANT: must come first before calling
                // another function that uses the token
                DEVICE_TOKEN = instanceIdResult.getToken();

                // Add onClick listener to "Answer Survey" button
                surveyButton.setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        fetchAssessment();
                    }
                });

                // Add onClick listener to "Refresh Status" button
                refreshButton.setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        fetchUserStatus();
                    }
                });

                // Fetch user status and assessment, if any
                fetchUserStatus();
//            fetchAssessment();

            }
        });

        return thisFragment;

    }

    private void fetchUserStatus() {

        // Get text fields and initialize other stuff
        final TextView statusTextView = this.thisFragment.findViewById(R.id.statusTextView);
        final TextView nextSurveyTextView = this.thisFragment.findViewById(R.id.nextSurveyTextView);
        final Button surveyButton = this.thisFragment.findViewById(R.id.surveyButton);
        final Button refreshButton = this.thisFragment.findViewById(R.id.refreshButton);

        // Change UI to indicate loading is happening
        statusTextView.setText(R.string.loadingText);
        nextSurveyTextView.setVisibility(View.GONE);

        // Define task
        FetchUserStatus userStatusTask = new FetchUserStatus(new URLEventListener() {

            @Override
            public void onSuccess(UserStatusModel userStatus) {
                if(userStatus.isEnrolled()) {

                    // Change next survey date text (as well as status text)
                    statusTextView.setText(userStatus.toString());
                    nextSurveyTextView.setText(userStatus.getTimeToNext());
                    nextSurveyTextView.setVisibility(View.VISIBLE);

                    // Hide refresh button (it only refreshes user status, anyways so...)
                    refreshButton.setVisibility(View.GONE);

                    // Toggle "Answer" button visibility depending on survey availability
                    if(userStatus.surveyAvailable()) {
                        surveyButton.setVisibility(View.VISIBLE);
                    } else {
                        surveyButton.setVisibility(View.GONE);
                    }

                } else {
                    // Prompt user to refresh status if user doesn't have an experience
//                    statusTextView.setText("You are not enrolled in any experience.\nIf you think this is a mistake, contact your supervisors."); // for debugging/testing
                    statusTextView.setText(userStatus.toString());
                    nextSurveyTextView.setVisibility(View.GONE);
                    surveyButton.setVisibility(View.GONE);
                    refreshButton.setVisibility(View.VISIBLE);
                }
            }

            @Override
            public void onFailure(Exception e) {
                statusTextView.setText(R.string.userStatusErrorText);
                Toast.makeText(getContext(), "Can't verify identity!", Toast.LENGTH_LONG).show();
                Log.e("ERROR WHILE FETCHING USER STATUS", e.toString());
            }

        });

        // Start task
        userStatusTask.execute(this.DEVICE_TOKEN);

    }

    private void fetchAssessment() {

        // Initiate progress dialog
        // TODO: find substitute for this deprecated dialog box
        final ProgressDialog progressDialog = ProgressDialog.show(getContext(),
                "Looking for Survey",
                "This shouldn't take long");

        // Fetch the assessment to be taken (if there is at least one available),
        // and launch the assessment (if the button is pressed)
        FetchAssessment assessmentTask = new FetchAssessment(new URLEventListener() {

            @Override
            public void onSuccess(AssessmentModel assessment) {
                progressDialog.dismiss();
                if(assessment.isEmpty()) {
                    fetchUserStatus(); // update status to let user know if it already had expired by the time he clicked
                    Toast.makeText(getContext(), "There are no surveys at the moment!", Toast.LENGTH_LONG).show();
                } else {
                    launchSurvey(assessment);
                }
            }

            @Override
            public void onFailure(Exception e) {
                progressDialog.dismiss();
                Toast.makeText(getContext(), "Can't reach surveys at the moment!", Toast.LENGTH_LONG).show();
                Log.e("ERROR WHILE FETCHING ASSESSMENT", e.toString());
            }

        });

        // Start task
        assessmentTask.execute(this.DEVICE_TOKEN);

    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if(requestCode == REQUEST_SURVEY && resultCode == Activity.RESULT_OK) {
            this.processSurveyResult((TaskResult) data.getSerializableExtra(ViewTaskActivity.EXTRA_TASK_RESULT));
        }
    }

    private void launchSurvey(AssessmentModel assessment) {
        this.ASSESSMENT_ID = assessment.getID();
        List<Step> steps = this.prepareSurveySteps(assessment);
        OrderedTask task = new OrderedTask(SAMPLE_SURVEY, steps);
        Intent intent = ViewTaskActivity.newIntent(getContext(), task);
        startActivityForResult(intent, REQUEST_SURVEY);
    }

    private void processSurveyResult(TaskResult result) {

        // Save results to local database
        StorageAccess.getInstance().getAppDatabase().saveTaskResult(result);

        // Load results
        TaskResult taskResult = StorageAccess.getInstance()
                .getAppDatabase()
                .loadLatestTaskResult(SAMPLE_SURVEY);

        // Prepare and send results
        AnsweredAssessmentModel answeredAssessment = this.prepareAnsweredAssessment(taskResult);
        this.sendSurveyInfo(answeredAssessment);

    }

    private void sendSurveyInfo(final AnsweredAssessmentModel answeredAssessment) {

        // Initiate progress dialog
        // TODO: find substitute for this deprecated dialog box
        final ProgressDialog progressDialog = ProgressDialog.show(getContext(),
                "Sending Answers",
                "This shouldn't take long");

        // Send answers and prompt user if he wants to retry sending them if an error occurs
        SendAnswers sendAnswersTask = new SendAnswers(new URLEventListener() {

           @Override public void onSuccess() {
               progressDialog.dismiss();
               Toast.makeText(getContext(), "Answers sent!", Toast.LENGTH_LONG).show();
               fetchUserStatus();
           }

           @Override
           public void onFailure(Exception e) {
               Log.e("ERROR WHILE SENDING ANSWERS", e.toString());
               e.printStackTrace();
               progressDialog.dismiss();
               promptRetrySend(answeredAssessment);
           }

        });


        // Prepare results and start task
       sendAnswersTask.execute(answeredAssessment);

        // UNCOMMENT THIS FOR TESTING (REMEMBER TO COMMENT .execute() LINE)
//        Handler handler = new Handler();
//        handler.postDelayed(new Runnable() {
//            public void run() {
//                progressDialog.dismiss();
//                if(true) { // Imitate success
//                    Toast.makeText(getContext(), "Answers sent!", Toast.LENGTH_LONG).show();
//                    fetchUserStatus();
//                } else { // Imitate failure
//                    promptRetrySend2(answeredAssessment);
//                }
//            }
//        }, 5000);
        // UNCOMMENT THIS FOR TESTING (REMEMBER TO COMMENT .execute() LINE)

    }

    private AnsweredAssessmentModel prepareAnsweredAssessment(TaskResult taskResult) {

        // Gather answered questions
        ArrayList<AnsweredQuestionModel> answeredQuestions = new ArrayList<>();
        for (String questionID : taskResult.getResults().keySet()) {

            // Extract question info
            // TODO: check if instead of getting the taskResult start/end times,
            //  we should be getting the stepResult start/end times...
            //  IF NOT: we should think about restructuring the JSON
            //  because we're being highly redundant and inefficient
            StepResult stepResult = taskResult.getStepResult(questionID);
            String answer = stepResult.getResult().toString();
            String startDatetime = taskResult.getStartDate().toString();
            String endDatetime = taskResult.getEndDate().toString();

            // Append question
            AnsweredQuestionModel answeredQuestion = new AnsweredQuestionModel(questionID, answer, startDatetime, endDatetime);
            answeredQuestions.add(answeredQuestion);

        }

        // Return answered assessment
        return new AnsweredAssessmentModel(this.ASSESSMENT_ID, this.DEVICE_TOKEN, answeredQuestions);

    }

    private List<Step> prepareSurveySteps(AssessmentModel assessment) {

        // Initialize survey steps
        List<Step> steps = new ArrayList<>();

        // Create instruction screen (add it to total steps)
        InstructionStep instructionStep = new InstructionStep(INSTRUCTION,
                getString(R.string.app_name),
                assessment.getDescription());
        instructionStep.setStepTitle(R.string.surveyToolbarText);
        steps.add(0, instructionStep);

        // Prepare survey question by question
        List<QuestionModel> questions = assessment.getQuestions();
        Log.d(TAG, "AssessmentCount: " + assessment.getQuestionCount() + " QuestionCount: " + questions.size()); // just to be sure
        for(int i = 0; i < assessment.getQuestionCount(); i++) {

            QuestionModel question = questions.get(i);
            Log.d(TAG, "QuestionType:" + question.getType());

            if(question.getType().equals("SCALED")) {

                // Create question choices
                Choice[] choices = new Choice[question.getMaxValue()];
                for(int m = 0; m < question.getMaxValue(); m++) {
                    if (m == 0) {
                        choices[m] = new Choice<>((m + 1) + " " + question.getMinText(), m + 1);
                    } else if (m == question.getMaxValue() - 1) {
                        choices[m] = new Choice<>((m + 1) + " " + question.getMaxText(), m + 1);
                    } else {
                        choices[m] = new Choice<>(Integer.toString(m + 1), m + 1);
                    }
                }

                // Create question step along with its choices
                QuestionStep newQuestionStep = new QuestionStep(question.getID());
                AnswerFormat multiFormat = new ChoiceAnswerFormat(AnswerFormat.ChoiceAnswerStyle.SingleChoice, choices);

                // Set question step details
                newQuestionStep.setStepTitle(R.string.surveyToolbarText);
                newQuestionStep.setTitle(question.getPremise());
                newQuestionStep.setAnswerFormat(multiFormat);
                newQuestionStep.setOptional(false);

                // Add step to arrays
                steps.add(newQuestionStep);

            } else if(question.getType().equals("OPEN")) {

                // Create question step along with its choices
                QuestionStep newQuestionStep = new QuestionStep(question.getID());
                TextAnswerFormat format = new TextAnswerFormat(100);
                format.setIsMultipleLines(true);

                // Set question step details
                newQuestionStep.setStepTitle(R.string.surveyToolbarText);
                newQuestionStep.setTitle(question.getPremise());
                newQuestionStep.setAnswerFormat(format);
                newQuestionStep.setOptional(false);

                // Add step to arrays
                steps.add(newQuestionStep);

            } else {
                Log.wtf(TAG, "Neither OPEN nor SCALED if-statement has been reached");
            }

        }

        return steps;
    }

    private void promptRetrySend(final AnsweredAssessmentModel answeredAssessment) {
        AlertDialog.Builder builder = new AlertDialog.Builder(Objects.requireNonNull(getContext()));
        builder.setCancelable(true);
        builder.setTitle("ERROR");
        builder.setMessage("Couldn't send answer!");
        builder.setPositiveButton("Retry", new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                Log.d(TAG, "Retrying answer send!");
                sendSurveyInfo(answeredAssessment);
            }
        });
        builder.setNegativeButton("Cancel", new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                Toast.makeText(getContext(), "Didn't send answers!", Toast.LENGTH_LONG).show();
                fetchUserStatus();
            }
        });

        AlertDialog dialog = builder.create();
        dialog.show();
    }

}
